print("import: 'cyvlfeat.fisher.fisher'")
import cyvlfeat.fisher.fisher

print("import: 'cyvlfeat.generic.generic'")
import cyvlfeat.generic.generic

print("import: 'cyvlfeat.hog.hog'")
import cyvlfeat.hog.hog

print("import: 'cyvlfeat.kmeans.kmeans'")
import cyvlfeat.kmeans.kmeans

print("import: 'cyvlfeat.quickshift.quickshift'")
import cyvlfeat.quickshift.quickshift

print("import: 'cyvlfeat.sift.dsift'")
import cyvlfeat.sift.dsift

print("import: 'cyvlfeat.sift.phow'")
import cyvlfeat.sift.phow

print("import: 'cyvlfeat.sift.sift'")
import cyvlfeat.sift.sift

print("import: 'cyvlfeat.vlad.vlad'")
import cyvlfeat.vlad.vlad

